import React, { useState, useMemo, useCallback, useRef, useLayoutEffect } from 'react';
import { Animated, PanResponder } from 'react-native';
import { useInterval } from '../hooks/use-interval.js';
import styles from '../styles.js';
import { SuccessToast } from './success.js';
import { WarningToast } from './warning.js';
import { ErrorToast } from './error.js';
import { InfoToast } from './info.js';
import { useToastContext } from '../contexts/toast-context.js';

const defaultComponentsConfig = {
    success: (props) => React.createElement(SuccessToast, Object.assign({}, props)),
    warning: (props) => React.createElement(WarningToast, Object.assign({}, props)),
    error: (props) => React.createElement(ErrorToast, Object.assign({}, props)),
    info: (props) => React.createElement(InfoToast, Object.assign({}, props)),
};
const ToasterInternal = () => {
    const { activeToast, defaults, customToasts, hideToast } = useToastContext();
    const toastTypes = {
        ...defaultComponentsConfig,
        ...customToasts,
    };
    const [inProgress, setInProgress] = useState(false);
    const [isVisible, setIsVisible] = useState(false);
    const [animation] = useState(new Animated.Value(0));
    const position = useMemo(() => activeToast?.position ?? defaults.position, [
        activeToast?.position,
        defaults.position,
    ]);
    const onHide = useMemo(() => activeToast?.onHide ?? defaults.onHide, [
        activeToast?.onHide,
        defaults.onHide,
    ]);
    const onShow = useMemo(() => activeToast?.onShow ?? defaults.onShow, [
        activeToast?.onShow,
        defaults.onShow,
    ]);
    const animateMovement = useCallback(({ dy }) => {
        let value = 1 + dy / 100;
        if (position === "bottom") {
            value = 1 - dy / 100;
        }
        if (value < 1) {
            animation?.setValue(value);
        }
    }, [animation, position]);
    const animateRelease = useCallback(({ dy, vy }) => {
        let value = 1 + dy / 100;
        if (position === "bottom") {
            value = 1 - dy / 100;
        }
        if (value < 0.65) {
            Animated.spring(animation, {
                toValue: -2,
                speed: position === "bottom" ? vy : -vy,
                useNativeDriver: true,
            }).start(() => {
                if (activeToast) {
                    onHide?.(activeToast);
                }
            });
        }
        else if (value < 0.95) {
            Animated.spring(animation, {
                toValue: 1,
                velocity: vy,
                useNativeDriver: true,
            }).start();
        }
    }, [activeToast, animation, onHide, position]);
    const friction = activeToast?.friction ?? defaults.friction;
    const animate = useCallback(({ toValue }) => {
        return new Promise((resolve) => {
            const spring = Animated.spring(animation, {
                toValue,
                friction,
                useNativeDriver: true,
            });
            spring.start(() => resolve(spring));
        });
    }, [animation, friction]);
    const animateShow = useCallback(() => {
        return animate({ toValue: 1 });
    }, [animate]);
    const animateHide = useCallback(() => {
        return animate({ toValue: 0 });
    }, [animate]);
    const panResponderRef = useRef(PanResponder.create({
        onStartShouldSetPanResponder: () => true,
        onPanResponderMove: (_event, gesture) => {
            animateMovement(gesture);
        },
        onPanResponderRelease: (_event, gesture) => {
            animateRelease(gesture);
        },
    }));
    const prevHeightRef = useRef();
    const heightRef = useRef(activeToast?.height ?? defaults.height);
    const onLayout = useCallback((e) => {
        prevHeightRef.current = heightRef.current;
        heightRef.current = e.nativeEvent.layout.height;
    }, []);
    const bottomOffset = activeToast?.bottomOffset ?? defaults.bottomOffset;
    const topOffset = activeToast?.topOffset ?? defaults.topOffset;
    const height = heightRef.current ?? defaults.height;
    const baseStyle = useMemo(() => {
        const offset = position === "bottom" ? bottomOffset : topOffset;
        const range = [height + 5, -offset];
        const outputRange = position === "bottom" ? range : range.map((i) => -i);
        const translateY = animation.interpolate({
            inputRange: [0, 1],
            outputRange,
        });
        return [
            styles.base,
            styles[position],
            {
                transform: [{ translateY }],
            },
        ];
    }, [animation, bottomOffset, height, position, topOffset]);
    const prevToastRef = useRef();
    useLayoutEffect(() => {
        if (activeToast === prevToastRef.current || (!activeToast && !prevToastRef.current)) {
            return;
        }
        const hide = async () => {
            setInProgress(true);
            await animateHide();
            setIsVisible(false);
            setInProgress(false);
            onHide?.(prevToastRef.current);
            prevToastRef.current = null;
        };
        const show = async () => {
            if (inProgress || isVisible) {
                await hide();
            }
            setInProgress(true);
            await animateShow();
            prevToastRef.current = activeToast;
            setIsVisible(true);
            setInProgress(false);
            onShow?.(activeToast);
        };
        if (!activeToast && isVisible && !inProgress) {
            hide();
        }
        if (activeToast && !isVisible && !inProgress) {
            show();
        }
        if (activeToast && !inProgress && activeToast !== prevToastRef.current) {
            show();
        }
    }, [activeToast, animateHide, animateShow, hideToast, inProgress, isVisible, onHide, onShow]);
    const autoHide = activeToast?.autoHide ?? defaults.autoHide;
    const visibilityTime = activeToast?.visibilityTime ?? defaults.visibilityTime;
    useInterval(hideToast, isVisible && autoHide && !inProgress ? visibilityTime : null);
    const toastType = activeToast?.type ?? defaults.type;
    const onPress = activeToast?.onPress ?? defaults.onPress;
    const onPressCallback = useMemo(() => {
        if (!onPress) {
            return undefined;
        }
        return (toast) => {
            hideToast();
            onPress?.(toast);
        };
    }, [hideToast, onPress]);
    const renderContent = () => {
        const toastComponent = toastTypes[toastType];
        if (!toastComponent) {
            console.error(`Toast of type '${toastType}' does not exist. Make sure to add it to your \`customToasts\`. You can read the documentation here: https://github.com/StratusCode/react-native-toast-hook/blob/master/README.md`);
            return null;
        }
        return toastComponent({
            ...defaults,
            ...activeToast,
            onClose: hideToast,
            onPress: onPressCallback,
        });
    };
    return (React.createElement(Animated.View, Object.assign({ onLayout: onLayout, style: baseStyle }, panResponderRef.current.panHandlers), renderContent()));
};
const Toaster = React.memo(ToasterInternal);

export { Toaster };
//# sourceMappingURL=Toaster.js.map
